@@ -19,10 +19,10 @@
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 # Standard Python modules
-import struct, math
-import numarray, FFT
+import os, sys, struct, math, struct
+import numarray,optparse, FFT
 
-__version__ = "$Revision: 1.4 $"
+__version__ = "$Revision: 1.3 $"
 __author__ = "Arnau Sanchez <arnau@ehas.org>"
 __depends__ = ['OSSAudioDev', 'FFT', 'Numeric-Extension', 'Python-2.4']
 __copyright__ = """Copyright (C) 2006 Arnau Sanchez <arnau@ehas.org>.
@@ -66,7 +66,7 @@ class Decoder:
 	SAMPLEFORMAT = {1: "b", 2: "h"}
 	
 	#########################
-	def __init__(self, samplerate=8000, samplewidth=2, mintime=0.5):
+	def __init__(self, samplerate, samplewidth, mintime):
 		if samplerate < self.MINSAMPLERATE: 
 			raise ValueError, "Samplerate must be %d sps or more: %s" %(self.MINSAMPLERATE, samplerate)
 		if samplewidth not in self.SAMPLEFORMAT:
@@ -87,18 +87,13 @@ class Decoder:
 		self.cosarray = {}
 		self.sinarray = {}
 		for freq in self.detect_tones:
-			k = 2*math.pi*freq/self.samplerate
-			self.sinarray[freq] = numarray.array([math.sin(k*x) for x in xrange(self.windowsize)])
-			self.cosarray[freq] = numarray.array([math.cos(k*x) for x in xrange(self.windowsize)])
+			self.sinarray[freq] = numarray.array([math.sin(2*math.pi*freq*x/self.samplerate) for x in range(self.windowsize)])
+			self.cosarray[freq] = numarray.array([math.cos(2*math.pi*freq*x/self.samplerate) for x in range(0, self.windowsize)])
 
 	#########################
 	def get_tone(self):
 		return self.tone_detected
 
-	#########################
-	def clear_tone(self):
-		self.tone_detected = self.tone_current = None
-
 	#########################
 	def decode_buffer(self, buffer):
 		self.buffer += buffer
@@ -149,7 +144,6 @@ class Decoder:
 
 ###########################
 def main():
-	import os, sys, optparse
 	usage = """
 	ctcss.py [options]: CTCSS Generator/Decoder
 	
