@@ -19,12 +19,13 @@
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 # Standard Python modules
-import os, sys, struct
-import math, numarray, optparse, FFT
+import os, sys, struct, operator
+import math, numarray,optparse, FFT
+import ossaudiodev, audioop
 
-__version__ = "$Revision: 1.6 $"
+__version__ = "$Revision: 1.5 $"
 __author__ = "Arnau Sanchez <arnau@ehas.org>"
-__depends__ = ['FFT', 'Numeric-Extension', 'Python-2.4']
+__depends__ = ['OSSAudioDev', 'FFT', 'Numeric-Extension', 'Python-2.4']
 __copyright__ = """Copyright (C) 2006 Arnau Sanchez <arnau@ehas.org>.
 This code is distributed under the terms of the GNU General Public License."""
 
@@ -46,13 +47,14 @@ MIN_TONETIME = 0.1
 SUBWINDOW = 2
 
 DEF_MIN_F1_POWER = 0.05
-DEF_DECODE_OVERPOWER = 20
-DEF_MIN_DIFF23_POWER = 10
-DEF_MAX_DIFF12_POWER = 20
+DEF_DECODE_OVERPOWER = 50
+DEF_MIN_DIFF23_POWER = 35
+DEF_MAX_DIFF12_POWER = 5
 
 PEAK_UP = 1
-PEAK_DOWN = 1
+PEAK_DOWN = 2
 
+# Common audio formats from ossaudiodev: 
 # dictionary: string_format : bit_order, sample size, signed/unsigned
 AFMT_TO_DEF = { "S8": "=bS", "U8": "=BU", "S16_LE": "<hS", "U16_LE": "<HU", "S16_BE": ">hS", "U16_BE": ">HU"}
 	
@@ -103,6 +105,7 @@ class Generator:
 		format = base_format.upper()
 		try: sampleformatdef = AFMT_TO_DEF[format]
 		except: raise NameError, "sample audio format not supported: %s" %base_format
+		self.sampleformat = getattr(ossaudiodev, "AFMT_" + format)
 		self.samplebyteorder, self.samplectype, self.samplesign  = sampleformatdef
 		self.float_to_audio = (1 << len(struct.pack(self.samplectype, 0)) * 8) / 2.0
 		self.audio_offset = 0
@@ -183,6 +186,7 @@ class Decoder:
 		format = base_format.upper()
 		try: sampleformatdef = AFMT_TO_DEF[format]
 		except: raise ValueError, "Audio format not supported: %s" %base_format
+		self.sampleformat = getattr(ossaudiodev, "AFMT_" + format)
 		self.samplebyteorder, self.samplectype, self.samplesign  = sampleformatdef
 		self.audio_to_float = 1.0 / float( (1 << len(struct.pack(self.samplectype, 0)) * 8) / 2.0 )
 		self.sample_length = len(struct.pack(self.samplectype, 0))
@@ -278,7 +282,6 @@ class Decoder:
 
 	################################################
 	def decode_buffer(self, buffer):
-		import audioop
 		if self.channels == 2:
 			buffer = audioop.tomono(buffer, self.sample_length, 0.5, 0.5)
 		format = self.samplebyteorder + str(len(buffer) / self.sample_length) + self.samplectype
@@ -312,10 +315,10 @@ def main():
 	parser.add_option('-s', '--samplerate', dest='samplerate', default = 8000, metavar='SPS', type='int', help = 'Set sampling rate')
 	parser.add_option('-f', '--sampleformat', dest='sampleformat', default = "S16_LE", metavar='AFMT_FORMAT', type='string', help = 'Set audio sample format')
 	parser.add_option('-c', '--channels', dest='channels', default = 1, metavar='NUMBER', type='int', help = 'Set audio channels')
-	parser.add_option('-e', '--sensibility', dest='sensibility', default = "1.0", metavar='VALUE', type='float', help = 'Decoding sensibility (1.0 for normal)')
+	parser.add_option('-e', '--sensibility', dest='sensibility', default = 1.0, metavar='VALUE', type='float', help = 'Decoding sensibility (1.0 for normal)')
 	parser.add_option('-g', '--generate', dest='generate', default = "", metavar='KEYS, TONETIME, WAITTIME, GAIN', type='string', help = 'DTMF generator')
 	parser.add_option('-d', '--decode', dest='decode', default = False, action='store_true', help = 'DTMF decoder')
-	parser.add_option('-b', '--buffersize', dest='buffersize', default = 1024, metavar = "BYTES", type = int, help = 'Buffer size for input/output')
+	parser.add_option('-b', '--buffersize', dest='buffersize', default = 1024, metavar = "BYTES", type = 'int', help = 'Buffer size for input/output')
 
 	options, args = parser.parse_args()
 	stdin, stdout = sys.stdin.fileno(), sys.stdout.fileno()
