@@ -19,20 +19,20 @@
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 # Standard Python modules
-import sys, time
+import sys, time, math
 import errno, audioop
+import struct
 
 # External phonepatch modules
-import soundcard
+import soundcard, ctcss
 
-__version__ = "$Revision: 1.1 $"
+__version__ = "$Revision: 1.8 $"
 __author__ = "Arnau Sanchez <arnau@ehas.org>"
 __depends__ = ['Python-2.4']
 __copyright__ = """Copyright (C) 2006 Arnau Sanchez <arnau@ehas.org>.
 This code is distributed under the terms of the GNU General Public License."""
 
-		
-###############################
+##############################
 ###############################
 class Radio:
 	"""Use soundcard and external PTT apps to interface with a radio transceiver.
@@ -44,55 +44,54 @@ class Radio:
 	"""
 
 	###############################
-	def __init__(self, soundcard_device, samplerate, buffer_samplerate, radio_control, ptt, carrier_detection, verbose = False, soundcard_retries = 1, **args):
+	def __init__(self, soundcard_device, samplerate, ptt, carrier, verbose=False, \
+		soundcard_retries=1, fullduplex=False, latency=None, ctcss_decoder=None):
 		"""Open a soundcard and PTT interface.
 
-		Use radio_control object to set PTT (method-name: set_ptt) and get 
-		carrier-detection state (method-name: get_carrier).
+		Use radio_control object to set PTT and get carrier-detection state.
 		
 		Soundcard device should be OSS files (/dev/dspX). Parameter 
-		<samplerate> will be the rate used by soundcard while 
-		<buffer_samplerate> stands for the rate of the read/write
-		buffer that it receives/send. Note that different samplerates
-		need a CPU-cost processing.
+		<samplerate> will be the rate used by soundcard.
 		
 		PTT object is an instance  of ExecInterface with "on" and "off"
 		commands defined.
 		"""
 		self.samplerate = samplerate
-		self.buffer_samplerate = buffer_samplerate
 		self.verbose = verbose
-		self.radio_control = radio_control
-		if self.radio_control:
-			self.ptt = ptt
-			self.carrier_detection = carrier_detection
-		else:
-			self.ptt = self.carrier_detection = False
-		
-		# PTT parameters
-		self.threshold_def = args.get("ptt_threshold_signal", 0.01)
-		self.tailtime_def = args.get("ptt_tail_time", 0)
-		self.maxtime_def = args.get("ptt_max_time", 120)
-		self.waittime_def = args.get("ptt_wait_time", 5)
+		self.ptt = ptt
+		self.carrier = carrier
 		
 		# Carrier parameters
-		self.fullduplex = args.get("fullduplex", False)
-		self.carrier_polling = args.get("carrier_polling", 0.1)
+		self.fullduplex = fullduplex
 		
 		# Soundcard parameters
-		self.sampleformat = "s16le"
+		self.sampleformat = "s16_le"
 		self.audio_channels = 1
-		self.buffer_size = 1024
+		self.buffer_size = 256
 		self.sample_width = 2
 		self.sample_max = 2.0**(2*8) / 2.0
 		
-		# Reduce soundcard fragment size (default is 4096), to reduce latency
-		self.fragment_size = 128
-		
+		# Latency (allowe dbetween 0.01 and secs) gives the fragment size 
+		self.fragmentsize = None
+		if latency:
+			self.fragmentsize = (samplerate * self.audio_channels * self.sample_width) * latency
+			if self.fragmentsize < 128: self.fragmentsize = 128
+			elif self.fragmentsize > 32768: self.fragmentsize = 32768
+			self.fragmentsize = 2**int(math.log(self.fragmentsize, 2))
+			self.debug("soundcard fragment size: %d bytes" %self.fragmentsize)
+			
 		self.onoff_dict = {False: "off", True: "on"}
-		self.offtime = self.ontime = self.tailtime = 0
-		self.carrier_state = False
-
+		self.ptt_offtime = self.ptt_ontime = self.ptt_tailtime = 0
+		self.carrier_offtime = self.carrier_ontime = self.carrier_tailtime = 0
+		self.carrier_state = None
+		self.set_carrier_state(False)
+	
+		# CTCSS generator/decoder		
+		if ctcss_decoder:
+			self.ctcss_generator = ctcss.Generator(self.samplerate, self.sample_width)
+			self.ctcss_decoder = ctcss.Decoder(self.samplerate, self.sample_width, ctcss_decoder)
+		else: self.ctcss_generator = self.ctcss_decoder = None
+		
 		# Open soundcard
 		self.soundcard = None
 		self.soundcard_device = soundcard_device
@@ -100,7 +99,7 @@ class Radio:
 			try: self.soundcard = soundcard.Soundcard(device = soundcard_device, \
 					channels = self.audio_channels, mode = "rw", library = "oss", \
 					samplerate = samplerate, sampleformat = self.sampleformat, \
-					fragment_size = self.fragment_size)
+					fragmentsize = self.fragmentsize)
 			except IOError, (nerror, detail): 
 				if nerror != errno.EBUSY: break
 				soundcard_retries -= 1
@@ -111,10 +110,6 @@ class Radio:
 				
 		if not self.soundcard:		
 			raise IOError, "cannot open soundcard: %s" %soundcard_device
-		
-		if samplerate != self.buffer_samplerate:
-			self.debug("warning: soundcard samplerate is set to %d, but output is %s" %(samplerate, self.buffer_samplerate))
-			self.debug("warning: resampling input/output audio with CPU penalty")
 			
 		# Turn PTT off at start (for safety)
 		self.set_ptt(False)
@@ -136,54 +131,58 @@ class Radio:
 		return self.soundcard
 
 	#####################################
-	def read_audio(self, size, resample = False):
-		"""Read data from soundcard 
-		
-		Resample audio buffer to <buffer_samplerate> if resampling 
-		enabled. If carrier detection is enabled, return audio buffer only
-		if carrier is detected."""
+	def read_audio(self, size):
+		"""Read data from soundcard""" 
 		if not self.soundcard: self.debug("soundcard not opened"); return
 		buffer = self.soundcard.read(size)
 		if not buffer: return
-			
-		# Return a void buffer if there is no carrier detection
-		if self.carrier_detection and not self.carrier_state:
-			return "\x00" * size
-		
-		# Resample if asked and if it's necessary
-		if resample and self.samplerate != self.buffer_samplerate:
-			buffer, state = audioop.ratecv(buffer, self.sample_width, self.audio_channels, self.samplerate, self.buffer_samplerate, self.rateread_state)
-			self.rateread_state = state
-			
+		buffer = self.update_carrier_state(buffer)
 		return buffer
 
 	#####################################
-	def update_carrier_state(self):
+	def decode_ctcss(self, buffer):
+		if not self.ctcss_decoder: return
+		self.ctcss_decoder.decode_buffer(buffer)
+		
+	#####################################
+	def get_ctcss_tone(self):
+		#return 200.0 # test
+		if not self.ctcss_decoder: return
+		return self.ctcss_decoder.get_tone()
+		
+	#####################################
+	def update_carrier_state(self, buffer):
 		"""Update carrier_detection state"""
-		if not self.carrier_detection: return
+		if not self.carrier: return buffer
+		if self.carrier.type == "audio": return buffer
 		try: next_time = self.time_next_carrier
 		except: next_time = 0
 		now = time.time()
 		if now > next_time:
-			try: self.carrier_state = self.radio_control.get_carrier()
-			except: self.debug("cannot get carrier state"); return
-			self.time_next_carrier = now + self.carrier_polling
-		try: old = self.old_carrier_state
-		except: old = None
-		if old != self.carrier_state:
-			self.debug("new carrier state: %s" %self.onoff_dict[self.carrier_state])
-			self.old_carrier_state = self.carrier_state
+			try: self.set_carrier_state(self.carrier.get())
+			except: self.debug("cannot get carrier state"); return buffer
+			self.time_next_carrier = now + self.carrier.pollingtime			
+		# Return a void buffer if there is no carrier detection
+		if self.carrier and self.carrier.type == "on" and not self.carrier_state:
+			return "\x00" * len(buffer)
+		return buffer
+			
+	########################################
+	def set_carrier_state(self, state):
+		if self.carrier_state != state:
+			self.debug("new carrier state: %s" %self.onoff_dict[state])
+			self.carrier_state = state
 		
 	########################################
 	def is_ptt_blocked(self):
 		"""Return a bool indicating if is possible to set the ptt on, 
 		otherwise the carrier detection is blocking it"""
-		if not self.carrier_detection or self.fullduplex or not self.carrier_state:
+		if not self.carrier or self.fullduplex or not self.carrier_state:
 			return False
 		return True
 		
 	#####################################
-	def vox_process(self, buffer, buffer_rate = 0):
+	def vox_toradio(self, buffer, ctcss=None):
 		"""VOX PTT processing.
 		
 		Set PTT on if audio data in buffer reaches the threshold. 
@@ -192,52 +191,96 @@ class Radio:
 		if not self.soundcard: raise IOError, "Soundcard not opened"
 		
 		if not self.ptt: 
-			self.send_audio(buffer, buffer_rate)
+			self.send_audio(buffer, ctcss)
 			return
 		
 		# Get power of audio fragment for VOX
 		power = audioop.rms(buffer, self.sample_width) / self.sample_max
-		
 		now = time.time()
 		
-		if power >= self.threshold_def and now >= self.ontime:
-			self.tailtime = now + self.tailtime_def
-			if not self.get_ptt():
-				self.debug("input power threshold reached: %0.4f" %self.threshold_def)
+		if power >= self.ptt.threshold and now >= self.ptt_ontime:
+			self.ptt_tailtime = now + self.ptt.tailtime
+			if not self.ptt.get():
+				self.debug("input power threshold reached: %0.4f" %self.ptt.threshold)
 				# Thereshold for PTT reached, but check before if not carrier is detected
 				if not self.is_ptt_blocked():
 					self.set_ptt(True)
-					self.ontime = 0
-					self.offtime = now + self.maxtime_def
-				else:
-					self.debug("PTT blocked due to carrier detection")
-			elif self.offtime and now >= self.offtime and not self.ontime:
-				self.debug("ptt_max_time timed out: turn PTT off and wait %d seconds" %self.waittime_def)
+					self.ptt_ontime = 0
+					if self.ptt.maxtime:
+						self.ptt_offtime = now + self.ptt.maxtime
+				else: self.debug("PTT blocked due to carrier detection")
+			elif self.ptt_offtime and now >= self.ptt_offtime and not self.ptt_ontime:
+				self.debug("ptt_max_time timed out: turn PTT off and wait %d seconds" %self.ptt.waittime)
+				self.set_ptt(False)
+				self.ptt_ontime = now + self.ptt.waittime
+			elif self.is_ptt_blocked():
+				self.debug("PTT blocked turned off due to carrier detection")
 				self.set_ptt(False)
-				self.ontime = now + self.waittime_def
 				
-		elif self.get_ptt():
-			if now >= self.tailtime:
+		elif self.ptt.get():
+			if now >= self.ptt_tailtime:
 				self.debug("ptt_tail_time reached")
 				self.set_ptt(False)
 			elif self.is_ptt_blocked():
 				self.debug("PTT blocked due to carrier detection")
 				self.set_ptt(False)
 
-		#if self.get_ptt():
-		self.send_audio(buffer, buffer_rate)
-			
+		#if self.ptt.get():
+		self.send_audio(buffer, ctcss)
+
 	#####################################
-	def send_audio(self, buffer, buffer_rate = 0):
-		"""Send audio to radio transceiver using the soundcard"""
+	def vox_topeer(self, peerfd, buffer):
+		"""VOX PTT processing.
+		
+		Set PTT on if audio data in buffer reaches the threshold. 
+		Control minimum and maximum PTT on/off states.
+		"""
+		if not self.carrier or self.carrier.type != "audio": 
+			peerfd.write(buffer)
+			peerfd.flush()
+			return
+		
+		# Get power of audio fragment for VOX
+		power = audioop.rms(buffer, self.sample_width) / self.sample_max
+		now = time.time()
+		
+		if power >= self.carrier.threshold and now >= self.carrier_ontime:
+			self.carrier_tailtime = now + self.carrier.tailtime
+			if not self.carrier_state:
+				self.debug("input power threshold for radio reached: %0.4f" %self.carrier.threshold)
+				if self.ptt.get(): self.set_ptt(False)
+				self.carrier_ontime = 0
+				if self.carrier.maxtime: 
+					self.carrier_offtime = now + self.carrier.maxtime
+				self.set_carrier_state(True)
+			elif self.carrier_offtime and now >= self.carrier_offtime and not self.carrier_ontime:
+				self.debug("carrier_max_time timed out: disabling carrier_detection for %d seconds" %self.ptt.waittime)
+				self.set_carrier_state(False)
+				self.carrier_ontime = now + self.carrier.waittime
+				
+		elif self.carrier_state and now >= self.carrier_tailtime:
+			self.debug("carrier_tail_time reached")
+			self.set_carrier_state(False)
+
+		#if self.ptt.get():
+		peerfd.write(buffer)
+		peerfd.flush()
+			
+
+	#####################################
+	def send_audio(self, buffer, ctcss=None):
+		"""Send audio to radio transceiver using the soundcard
+		
+		ctcss -- Tuple containing (frequecy, amplitude) for CTCSS code generation
+		"""
 		if not self.soundcard: self.debug("soundcard not opened"); return
 		if not buffer: return
-		
-		# Resample if asked and if it's necessary
-		if buffer_rate and buffer_rate != self.samplerate:
-			buffer, state = audioop.ratecv(buffer, self.sample_width, self.audio_channels, buffer_rate, self.samplerate, self.ratewrite_state)
-			self.ratewrite_state = state			
-		
+				
+		if ctcss and self.ctcss_generator:
+			freq, amplitude = ctcss
+			ctcss_buffer = self.ctcss_generator.generate(len(buffer), amplitude, freq)
+			buffer = audioop.add(buffer, ctcss_buffer, self.sample_width)
+
 		self.soundcard.write(buffer)
 
 	#####################################
@@ -254,24 +297,12 @@ class Radio:
 			self.soundcard.close()
 			self.soundcard = None
 			self.debug("soundcard closed")
-		else:
-			self.debug("soundcard was not opened")
+		else: self.debug("soundcard was not opened")
 		
-		if self.radio_control: 
-			self.debug("closing radio-control interface")
-			self.set_ptt(False)
-			self.radio_control.close()
+		self.set_ptt(False)
 
 	###################################
-	def set_ptt(self, state):
-		"""Set PTT state (bool)"""
+	def set_ptt(self, value):
 		if not self.ptt: return
-		self.debug("Setting PTT %s" %self.onoff_dict[state])
-		try: self.radio_control.set_ptt(state)
-		except: self.debug("error setting ptt")
-
-	###################################
-	def get_ptt(self):
-		"""Get PTT state -> bool"""
-		if not self.ptt: return False
-		return self.radio_control.get_ptt()
+		self.debug("set PTT: %s" %self.onoff_dict[bool(value)])
+		self.ptt.set(value)
